@@ -1,11 +1,11 @@
 // 2005-06-07 Benjamin Kosnik  <bkoz@redhat.com>
 
-// Copyright (C) 2005 Free Software Foundation, Inc.
+// Copyright (C) 2005, 2009 Free Software Foundation, Inc.
 //
 // This file is part of the GNU ISO C++ Library.  This library is free
 // software; you can redistribute it and/or modify it under the
 // terms of the GNU General Public License as published by the
-// Free Software Foundation; either version 2, or (at your option)
+// Free Software Foundation; either version 3, or (at your option)
 // any later version.
 
 // This library is distributed in the hope that it will be useful,
@@ -14,9 +14,8 @@
 // GNU General Public License for more details.
 
 // You should have received a copy of the GNU General Public License along
-// with this library; see the file COPYING.  If not, write to the Free
-// Software Foundation, 59 Temple Place - Suite 330, Boston, MA 02111-1307,
-// USA.
+// with this library; see the file COPYING3.  If not see
+// <http://www.gnu.org/licenses/>.
 
 // 27.8.1.4 Overridden virtual functions
 
@@ -39,7 +38,7 @@ void test05()
   typedef size_t 			size_type;
 
   bool test __attribute__((unused)) = true;
-  std::streamoff  			strmof_1, strmof_2;
+  std::streamoff  			strmof_1;
   testbuf	sb01;
 
   // int in_avail()
